
From: Ingo Molnar <mingo@elte.hu>

This patch implements a nu] mber of smp_processor_id() cleanup ideas that 
Arjan van de Ven and i came up with.

The previous __smp_processor_id/_smp_processor_id/smp_processor_id API 
spaghetti was hard to follow both on the implementational and on the 
usage side.

Some of the complexity arose from picking wrong names, some of the
complexity comes from the fact that not all architectures defined
__smp_processor_id.

In the new code, there are two externally visible symbols:

 - smp_processor_id(): debug variant.

 - raw_smp_processor_id(): nondebug variant. Replaces all existing
   uses of _smp_processor_id() and __smp_processor_id(). Defined
   by every SMP architecture in include/asm-*/smp.h.

There is one new internal symbol, dependent on DEBUG_PREEMPT:

 - debug_smp_processor_id(): internal debug variant, mapped to
                             smp_processor_id().

Also, i moved debug_smp_processor_id() from lib/kernel_lock.c into a new
lib/smp_processor_id.c file.  All related comments got updated and/or
clarified.

I have build/boot tested the following 8 .config combinations on x86:

 {SMP,UP} x {PREEMPT,!PREEMPT} x {DEBUG_PREEMPT,!DEBUG_PREEMPT}

I have also build/boot tested x64 on UP/PREEMPT/DEBUG_PREEMPT.  (Other
architectures are untested, but should work just fine.)

Signed-off-by: Ingo Molnar <mingo@elte.hu>
Signed-off-by: Arjan van de Ven <arjan@infradead.org>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 arch/i386/kernel/traps.c          |    2 -
 arch/i386/lib/delay.c             |    2 -
 arch/ppc/lib/locks.c              |    4 +-
 arch/ppc64/kernel/idle.c          |    2 -
 arch/sh/lib/delay.c               |    2 -
 arch/sparc64/lib/delay.c          |    2 -
 arch/x86_64/lib/delay.c           |    2 -
 drivers/acpi/processor_idle.c     |    2 -
 drivers/input/gameport/gameport.c |    2 -
 drivers/oprofile/buffer_sync.c    |    4 +-
 fs/xfs/linux-2.6/xfs_linux.h      |    6 ++--
 include/asm-alpha/smp.h           |    2 -
 include/asm-arm/smp.h             |    2 -
 include/asm-i386/smp.h            |    2 -
 include/asm-ia64/smp.h            |    2 -
 include/asm-m32r/smp.h            |    2 -
 include/asm-mips/smp.h            |    2 -
 include/asm-parisc/smp.h          |    2 -
 include/asm-ppc/smp.h             |    2 -
 include/asm-ppc64/smp.h           |    2 -
 include/asm-s390/smp.h            |    2 -
 include/asm-sh/smp.h              |    2 -
 include/asm-sparc/smp.h           |    2 -
 include/asm-sparc64/smp.h         |    2 -
 include/asm-um/smp.h              |    3 +-
 include/asm-x86_64/smp.h          |    2 -
 include/linux/mmzone.h            |    2 -
 include/linux/smp.h               |   40 +++++++++++----------------
 include/net/route.h               |    2 -
 include/net/snmp.h                |   14 ++++-----
 kernel/module.c                   |    2 -
 kernel/power/smp.c                |    4 +-
 kernel/sched.c                    |    4 +-
 kernel/stop_machine.c             |    4 +-
 lib/Makefile                      |    1 
 lib/kernel_lock.c                 |   55 --------------------------------------
 lib/smp_processor_id.c            |   55 ++++++++++++++++++++++++++++++++++++++
 37 files changed, 119 insertions(+), 125 deletions(-)

diff -puN arch/i386/kernel/traps.c~smp_processor_id-cleanup arch/i386/kernel/traps.c
--- 25/arch/i386/kernel/traps.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/arch/i386/kernel/traps.c	2005-06-18 22:32:07.000000000 -0700
@@ -306,7 +306,7 @@ void die(const char * str, struct pt_reg
 	};
 	static int die_counter;
 
-	if (die.lock_owner != _smp_processor_id()) {
+	if (die.lock_owner != raw_smp_processor_id()) {
 		console_verbose();
 		spin_lock_irq(&die.lock);
 		die.lock_owner = smp_processor_id();
diff -puN arch/i386/lib/delay.c~smp_processor_id-cleanup arch/i386/lib/delay.c
--- 25/arch/i386/lib/delay.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/arch/i386/lib/delay.c	2005-06-18 22:32:07.000000000 -0700
@@ -34,7 +34,7 @@ inline void __const_udelay(unsigned long
 	xloops *= 4;
 	__asm__("mull %0"
 		:"=d" (xloops), "=&a" (d0)
-		:"1" (xloops),"0" (cpu_data[_smp_processor_id()].loops_per_jiffy * (HZ/4)));
+		:"1" (xloops),"0" (cpu_data[raw_smp_processor_id()].loops_per_jiffy * (HZ/4)));
         __delay(++xloops);
 }
 
diff -puN arch/ppc64/kernel/idle.c~smp_processor_id-cleanup arch/ppc64/kernel/idle.c
--- 25/arch/ppc64/kernel/idle.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/arch/ppc64/kernel/idle.c	2005-06-18 22:32:07.000000000 -0700
@@ -292,7 +292,7 @@ static int native_idle(void)
 		if (need_resched())
 			schedule();
 
-		if (cpu_is_offline(_smp_processor_id()) &&
+		if (cpu_is_offline(raw_smp_processor_id()) &&
 		    system_state == SYSTEM_RUNNING)
 			cpu_die();
 	}
diff -puN arch/ppc/lib/locks.c~smp_processor_id-cleanup arch/ppc/lib/locks.c
--- 25/arch/ppc/lib/locks.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/arch/ppc/lib/locks.c	2005-06-18 22:32:07.000000000 -0700
@@ -130,7 +130,7 @@ void _raw_read_lock(rwlock_t *rw)
 		while (!read_can_lock(rw)) {
 			if (--stuck == 0) {
 				printk("_read_lock(%p) CPU#%d lock %d\n",
-				       rw, _smp_processor_id(), rw->lock);
+				       rw, raw_smp_processor_id(), rw->lock);
 				stuck = INIT_STUCK;
 			}
 		}
@@ -158,7 +158,7 @@ void _raw_write_lock(rwlock_t *rw)
 		while (!write_can_lock(rw)) {
 			if (--stuck == 0) {
 				printk("write_lock(%p) CPU#%d lock %d)\n",
-				       rw, _smp_processor_id(), rw->lock);
+				       rw, raw_smp_processor_id(), rw->lock);
 				stuck = INIT_STUCK;
 			}
 		}
diff -puN arch/sh/lib/delay.c~smp_processor_id-cleanup arch/sh/lib/delay.c
--- 25/arch/sh/lib/delay.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/arch/sh/lib/delay.c	2005-06-18 22:32:07.000000000 -0700
@@ -24,7 +24,7 @@ inline void __const_udelay(unsigned long
 	__asm__("dmulu.l	%0, %2\n\t"
 		"sts	mach, %0"
 		: "=r" (xloops)
-		: "0" (xloops), "r" (cpu_data[_smp_processor_id()].loops_per_jiffy)
+		: "0" (xloops), "r" (cpu_data[raw_smp_processor_id()].loops_per_jiffy)
 		: "macl", "mach");
 	__delay(xloops * HZ);
 }
diff -puN arch/sparc64/lib/delay.c~smp_processor_id-cleanup arch/sparc64/lib/delay.c
--- 25/arch/sparc64/lib/delay.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/arch/sparc64/lib/delay.c	2005-06-18 22:32:07.000000000 -0700
@@ -31,7 +31,7 @@ void __const_udelay(unsigned long n)
 {
 	n *= 4;
 
-	n *= (cpu_data(_smp_processor_id()).udelay_val * (HZ/4));
+	n *= (cpu_data(raw_smp_processor_id()).udelay_val * (HZ/4));
 	n >>= 32;
 
 	__delay(n + 1);
diff -puN arch/x86_64/lib/delay.c~smp_processor_id-cleanup arch/x86_64/lib/delay.c
--- 25/arch/x86_64/lib/delay.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/arch/x86_64/lib/delay.c	2005-06-18 22:32:07.000000000 -0700
@@ -34,7 +34,7 @@ void __delay(unsigned long loops)
 
 inline void __const_udelay(unsigned long xloops)
 {
-	__delay(((xloops * cpu_data[_smp_processor_id()].loops_per_jiffy) >> 32) * HZ);
+	__delay(((xloops * cpu_data[raw_smp_processor_id()].loops_per_jiffy) >> 32) * HZ);
 }
 
 void __udelay(unsigned long usecs)
diff -puN drivers/acpi/processor_idle.c~smp_processor_id-cleanup drivers/acpi/processor_idle.c
--- 25/drivers/acpi/processor_idle.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/drivers/acpi/processor_idle.c	2005-06-18 22:32:07.000000000 -0700
@@ -171,7 +171,7 @@ static void acpi_processor_idle (void)
 	int			sleep_ticks = 0;
 	u32			t1, t2 = 0;
 
-	pr = processors[_smp_processor_id()];
+	pr = processors[raw_smp_processor_id()];
 	if (!pr)
 		return;
 
diff -puN drivers/input/gameport/gameport.c~smp_processor_id-cleanup drivers/input/gameport/gameport.c
--- 25/drivers/input/gameport/gameport.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/drivers/input/gameport/gameport.c	2005-06-18 22:32:07.000000000 -0700
@@ -134,7 +134,7 @@ static int gameport_measure_speed(struct
 	}
 
 	gameport_close(gameport);
-	return (cpu_data[_smp_processor_id()].loops_per_jiffy * (unsigned long)HZ / (1000 / 50)) / (tx < 1 ? 1 : tx);
+	return (cpu_data[raw_smp_processor_id()].loops_per_jiffy * (unsigned long)HZ / (1000 / 50)) / (tx < 1 ? 1 : tx);
 
 #else
 
diff -puN drivers/oprofile/buffer_sync.c~smp_processor_id-cleanup drivers/oprofile/buffer_sync.c
--- 25/drivers/oprofile/buffer_sync.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/drivers/oprofile/buffer_sync.c	2005-06-18 22:32:07.000000000 -0700
@@ -62,7 +62,7 @@ static int task_exit_notify(struct notif
 	/* To avoid latency problems, we only process the current CPU,
 	 * hoping that most samples for the task are on this CPU
 	 */
-	sync_buffer(_smp_processor_id());
+	sync_buffer(raw_smp_processor_id());
   	return 0;
 }
 
@@ -86,7 +86,7 @@ static int munmap_notify(struct notifier
 		/* To avoid latency problems, we only process the current CPU,
 		 * hoping that most samples for the task are on this CPU
 		 */
-		sync_buffer(_smp_processor_id());
+		sync_buffer(raw_smp_processor_id());
 		return 0;
 	}
 
diff -puN fs/xfs/linux-2.6/xfs_linux.h~smp_processor_id-cleanup fs/xfs/linux-2.6/xfs_linux.h
--- 25/fs/xfs/linux-2.6/xfs_linux.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/fs/xfs/linux-2.6/xfs_linux.h	2005-06-18 22:32:07.000000000 -0700
@@ -145,10 +145,10 @@ static inline void set_buffer_unwritten_
 #define xfs_inherit_nosymlinks	xfs_params.inherit_nosym.val
 #define xfs_rotorstep		xfs_params.rotorstep.val
 
-#ifndef __smp_processor_id
-#define __smp_processor_id()	smp_processor_id()
+#ifndef raw_smp_processor_id
+#define raw_smp_processor_id()	smp_processor_id()
 #endif
-#define current_cpu()		__smp_processor_id()
+#define current_cpu()		raw_smp_processor_id()
 #define current_pid()		(current->pid)
 #define current_fsuid(cred)	(current->fsuid)
 #define current_fsgid(cred)	(current->fsgid)
diff -puN include/asm-alpha/smp.h~smp_processor_id-cleanup include/asm-alpha/smp.h
--- 25/include/asm-alpha/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-alpha/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -43,7 +43,7 @@ extern struct cpuinfo_alpha cpu_data[NR_
 #define PROC_CHANGE_PENALTY     20
 
 #define hard_smp_processor_id()	__hard_smp_processor_id()
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id()	(current_thread_info()->cpu)
 
 extern cpumask_t cpu_present_mask;
 extern cpumask_t cpu_online_map;
diff -puN include/asm-arm/smp.h~smp_processor_id-cleanup include/asm-arm/smp.h
--- 25/include/asm-arm/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-arm/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -21,7 +21,7 @@
 # error "<asm-arm/smp.h> included in non-SMP build"
 #endif
 
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id() (current_thread_info()->cpu)
 
 extern cpumask_t cpu_present_mask;
 #define cpu_possible_map cpu_present_mask
diff -puN include/asm-i386/smp.h~smp_processor_id-cleanup include/asm-i386/smp.h
--- 25/include/asm-i386/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-i386/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -51,7 +51,7 @@ extern u8 x86_cpu_to_apicid[];
  * from the initial startup. We map APIC_BASE very early in page_setup(),
  * so this is correct in the x86 case.
  */
-#define __smp_processor_id() (current_thread_info()->cpu)
+#define raw_smp_processor_id() (current_thread_info()->cpu)
 
 extern cpumask_t cpu_callout_map;
 extern cpumask_t cpu_callin_map;
diff -puN include/asm-ia64/smp.h~smp_processor_id-cleanup include/asm-ia64/smp.h
--- 25/include/asm-ia64/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-ia64/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -46,7 +46,7 @@ ia64_get_lid (void)
 #define SMP_IRQ_REDIRECTION	(1 << 0)
 #define SMP_IPI_REDIRECTION	(1 << 1)
 
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id() (current_thread_info()->cpu)
 
 extern struct smp_boot_data {
 	int cpu_count;
diff -puN include/asm-m32r/smp.h~smp_processor_id-cleanup include/asm-m32r/smp.h
--- 25/include/asm-m32r/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-m32r/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -66,7 +66,7 @@ extern volatile int cpu_2_physid[NR_CPUS
 #define physid_to_cpu(physid)	physid_2_cpu[physid]
 #define cpu_to_physid(cpu_id)	cpu_2_physid[cpu_id]
 
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id()	(current_thread_info()->cpu)
 
 extern cpumask_t cpu_callout_map;
 #define cpu_possible_map cpu_callout_map
diff -puN include/asm-mips/smp.h~smp_processor_id-cleanup include/asm-mips/smp.h
--- 25/include/asm-mips/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-mips/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -21,7 +21,7 @@
 #include <linux/cpumask.h>
 #include <asm/atomic.h>
 
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id() (current_thread_info()->cpu)
 
 /* Map from cpu id to sequential logical cpu number.  This will only
    not be idempotent when cpus failed to come on-line.  */
diff -puN include/asm-parisc/smp.h~smp_processor_id-cleanup include/asm-parisc/smp.h
--- 25/include/asm-parisc/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-parisc/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -51,7 +51,7 @@ extern void smp_send_reschedule(int cpu)
 
 extern unsigned long cpu_present_mask;
 
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id()	(current_thread_info()->cpu)
 
 #endif /* CONFIG_SMP */
 
diff -puN include/asm-ppc64/smp.h~smp_processor_id-cleanup include/asm-ppc64/smp.h
--- 25/include/asm-ppc64/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-ppc64/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -45,7 +45,7 @@ void generic_cpu_die(unsigned int cpu);
 void generic_mach_cpu_die(void);
 #endif
 
-#define __smp_processor_id() (get_paca()->paca_index)
+#define raw_smp_processor_id()	(get_paca()->paca_index)
 #define hard_smp_processor_id() (get_paca()->hw_cpu_id)
 
 extern cpumask_t cpu_sibling_map[NR_CPUS];
diff -puN include/asm-ppc/smp.h~smp_processor_id-cleanup include/asm-ppc/smp.h
--- 25/include/asm-ppc/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-ppc/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -44,7 +44,7 @@ extern void smp_message_recv(int, struct
 #define NO_PROC_ID		0xFF            /* No processor magic marker */
 #define PROC_CHANGE_PENALTY	20
 
-#define smp_processor_id() (current_thread_info()->cpu)
+#define raw_smp_processor_id()	(current_thread_info()->cpu)
 
 extern int __cpu_up(unsigned int cpu);
 
diff -puN include/asm-s390/smp.h~smp_processor_id-cleanup include/asm-s390/smp.h
--- 25/include/asm-s390/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-s390/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -47,7 +47,7 @@ extern int smp_call_function_on(void (*f
  
 #define PROC_CHANGE_PENALTY	20		/* Schedule penalty */
 
-#define smp_processor_id() (S390_lowcore.cpu_data.cpu_nr)
+#define raw_smp_processor_id()	(S390_lowcore.cpu_data.cpu_nr)
 
 extern int smp_get_cpu(cpumask_t cpu_map);
 extern void smp_put_cpu(int cpu);
diff -puN include/asm-sh/smp.h~smp_processor_id-cleanup include/asm-sh/smp.h
--- 25/include/asm-sh/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-sh/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -25,7 +25,7 @@ extern cpumask_t cpu_possible_map;
 
 #define cpu_online(cpu)		cpu_isset(cpu, cpu_online_map)
 
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id()	(current_thread_info()->cpu)
 
 /* I've no idea what the real meaning of this is */
 #define PROC_CHANGE_PENALTY	20
diff -puN include/asm-sparc64/smp.h~smp_processor_id-cleanup include/asm-sparc64/smp.h
--- 25/include/asm-sparc64/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-sparc64/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -64,7 +64,7 @@ static __inline__ int hard_smp_processor
 	}
 }
 
-#define smp_processor_id() (current_thread_info()->cpu)
+#define raw_smp_processor_id() (current_thread_info()->cpu)
 
 #endif /* !(__ASSEMBLY__) */
 
diff -puN include/asm-sparc/smp.h~smp_processor_id-cleanup include/asm-sparc/smp.h
--- 25/include/asm-sparc/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-sparc/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -148,7 +148,7 @@ extern __inline__ int hard_smp_processor
 }
 #endif
 
-#define smp_processor_id()	(current_thread_info()->cpu)
+#define raw_smp_processor_id()		(current_thread_info()->cpu)
 
 #define prof_multiplier(__cpu)		cpu_data(__cpu).multiplier
 #define prof_counter(__cpu)		cpu_data(__cpu).counter
diff -puN include/asm-um/smp.h~smp_processor_id-cleanup include/asm-um/smp.h
--- 25/include/asm-um/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-um/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -8,7 +8,8 @@
 #include "asm/current.h"
 #include "linux/cpumask.h"
 
-#define smp_processor_id() (current_thread->cpu)
+#define raw_smp_processor_id() (current_thread->cpu)
+
 #define cpu_logical_map(n) (n)
 #define cpu_number_map(n) (n)
 #define PROC_CHANGE_PENALTY	15 /* Pick a number, any number */
diff -puN include/asm-x86_64/smp.h~smp_processor_id-cleanup include/asm-x86_64/smp.h
--- 25/include/asm-x86_64/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/asm-x86_64/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -68,7 +68,7 @@ static inline int num_booting_cpus(void)
 	return cpus_weight(cpu_callout_map);
 }
 
-#define __smp_processor_id() read_pda(cpunumber)
+#define raw_smp_processor_id() read_pda(cpunumber)
 
 extern __inline int hard_smp_processor_id(void)
 {
diff -puN include/linux/mmzone.h~smp_processor_id-cleanup include/linux/mmzone.h
--- 25/include/linux/mmzone.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/linux/mmzone.h	2005-06-18 22:32:07.000000000 -0700
@@ -381,7 +381,7 @@ int lowmem_reserve_ratio_sysctl_handler(
 
 #include <linux/topology.h>
 /* Returns the number of the current Node. */
-#define numa_node_id()		(cpu_to_node(_smp_processor_id()))
+#define numa_node_id()		(cpu_to_node(raw_smp_processor_id()))
 
 #ifndef CONFIG_DISCONTIGMEM
 
diff -puN include/linux/smp.h~smp_processor_id-cleanup include/linux/smp.h
--- 25/include/linux/smp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/linux/smp.h	2005-06-18 22:32:07.000000000 -0700
@@ -92,10 +92,7 @@ void smp_prepare_boot_cpu(void);
 /*
  *	These macros fold the SMP functionality into a single CPU system
  */
-
-#if !defined(__smp_processor_id) || !defined(CONFIG_PREEMPT)
-# define smp_processor_id()			0
-#endif
+#define raw_smp_processor_id()			0
 #define hard_smp_processor_id()			0
 #define smp_call_function(func,info,retry,wait)	({ 0; })
 #define on_each_cpu(func,info,retry,wait)	({ func(info); 0; })
@@ -106,30 +103,25 @@ static inline void smp_send_reschedule(i
 #endif /* !SMP */
 
 /*
- * DEBUG_PREEMPT support: check whether smp_processor_id() is being
- * used in a preemption-safe way.
+ * smp_processor_id(): get the current CPU ID.
  *
- * An architecture has to enable this debugging code explicitly.
- * It can do so by renaming the smp_processor_id() macro to
- * __smp_processor_id().  This should only be done after some minimal
- * testing, because usually there are a number of false positives
- * that an architecture will trigger.
+ * if DEBUG_PREEMPT is enabled the we check whether it is
+ * used in a preemption-safe way. (smp_processor_id() is safe
+ * if it's used in a preemption-off critical section, or in
+ * a thread that is bound to the current CPU.)
  *
- * To fix a false positive (i.e. smp_processor_id() use that the
- * debugging code reports but which use for some reason is legal),
- * change the smp_processor_id() reference to _smp_processor_id(),
- * which is the nondebug variant.  NOTE: don't use this to hack around
- * real bugs.
+ * NOTE: raw_smp_processor_id() is for internal use only
+ * (smp_processor_id() is the preferred variant), but in rare
+ * instances it might also be used to turn off false positives
+ * (i.e. smp_processor_id() use that the debugging code reports but
+ * which use for some reason is legal). Don't use this to hack around
+ * the warning message, as your code might not work under PREEMPT.
  */
-#ifdef __smp_processor_id
-# if defined(CONFIG_PREEMPT) && defined(CONFIG_DEBUG_PREEMPT)
-   extern unsigned int smp_processor_id(void);
-# else
-#  define smp_processor_id() __smp_processor_id()
-# endif
-# define _smp_processor_id() __smp_processor_id()
+#ifdef CONFIG_DEBUG_PREEMPT
+  extern unsigned int debug_smp_processor_id(void);
+# define smp_processor_id() debug_smp_processor_id()
 #else
-# define _smp_processor_id() smp_processor_id()
+# define smp_processor_id() raw_smp_processor_id()
 #endif
 
 #define get_cpu()		({ preempt_disable(); smp_processor_id(); })
diff -puN include/net/route.h~smp_processor_id-cleanup include/net/route.h
--- 25/include/net/route.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/net/route.h	2005-06-18 22:32:07.000000000 -0700
@@ -107,7 +107,7 @@ struct rt_cache_stat 
 
 extern struct rt_cache_stat *rt_cache_stat;
 #define RT_CACHE_STAT_INC(field)					  \
-		(per_cpu_ptr(rt_cache_stat, _smp_processor_id())->field++)
+		(per_cpu_ptr(rt_cache_stat, raw_smp_processor_id())->field++)
 
 extern struct ip_rt_acct *ip_rt_acct;
 
diff -puN include/net/snmp.h~smp_processor_id-cleanup include/net/snmp.h
--- 25/include/net/snmp.h~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/include/net/snmp.h	2005-06-18 22:32:07.000000000 -0700
@@ -128,18 +128,18 @@ struct linux_mib {
 #define SNMP_STAT_USRPTR(name)	(name[1])
 
 #define SNMP_INC_STATS_BH(mib, field) 	\
-	(per_cpu_ptr(mib[0], _smp_processor_id())->mibs[field]++)
+	(per_cpu_ptr(mib[0], raw_smp_processor_id())->mibs[field]++)
 #define SNMP_INC_STATS_OFFSET_BH(mib, field, offset)	\
-	(per_cpu_ptr(mib[0], _smp_processor_id())->mibs[field + (offset)]++)
+	(per_cpu_ptr(mib[0], raw_smp_processor_id())->mibs[field + (offset)]++)
 #define SNMP_INC_STATS_USER(mib, field) \
-	(per_cpu_ptr(mib[1], _smp_processor_id())->mibs[field]++)
+	(per_cpu_ptr(mib[1], raw_smp_processor_id())->mibs[field]++)
 #define SNMP_INC_STATS(mib, field) 	\
-	(per_cpu_ptr(mib[!in_softirq()], _smp_processor_id())->mibs[field]++)
+	(per_cpu_ptr(mib[!in_softirq()], raw_smp_processor_id())->mibs[field]++)
 #define SNMP_DEC_STATS(mib, field) 	\
-	(per_cpu_ptr(mib[!in_softirq()], _smp_processor_id())->mibs[field]--)
+	(per_cpu_ptr(mib[!in_softirq()], raw_smp_processor_id())->mibs[field]--)
 #define SNMP_ADD_STATS_BH(mib, field, addend) 	\
-	(per_cpu_ptr(mib[0], _smp_processor_id())->mibs[field] += addend)
+	(per_cpu_ptr(mib[0], raw_smp_processor_id())->mibs[field] += addend)
 #define SNMP_ADD_STATS_USER(mib, field, addend) 	\
-	(per_cpu_ptr(mib[1], _smp_processor_id())->mibs[field] += addend)
+	(per_cpu_ptr(mib[1], raw_smp_processor_id())->mibs[field] += addend)
 
 #endif
diff -puN kernel/module.c~smp_processor_id-cleanup kernel/module.c
--- 25/kernel/module.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/kernel/module.c	2005-06-18 22:32:07.000000000 -0700
@@ -379,7 +379,7 @@ static void module_unload_init(struct mo
 	for (i = 0; i < NR_CPUS; i++)
 		local_set(&mod->ref[i].count, 0);
 	/* Hold reference count during initialization. */
-	local_set(&mod->ref[_smp_processor_id()].count, 1);
+	local_set(&mod->ref[raw_smp_processor_id()].count, 1);
 	/* Backwards compatibility macros put refcount during init. */
 	mod->waiter = current;
 }
diff -puN kernel/power/smp.c~smp_processor_id-cleanup kernel/power/smp.c
--- 25/kernel/power/smp.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/kernel/power/smp.c	2005-06-18 22:32:07.000000000 -0700
@@ -48,11 +48,11 @@ void disable_nonboot_cpus(void)
 {
 	oldmask = current->cpus_allowed;
 	set_cpus_allowed(current, cpumask_of_cpu(0));
-	printk("Freezing CPUs (at %d)", _smp_processor_id());
+	printk("Freezing CPUs (at %d)", raw_smp_processor_id());
 	current->state = TASK_INTERRUPTIBLE;
 	schedule_timeout(HZ);
 	printk("...");
-	BUG_ON(_smp_processor_id() != 0);
+	BUG_ON(raw_smp_processor_id() != 0);
 
 	/* FIXME: for this to work, all the CPUs must be running
 	 * "idle" thread (or we deadlock). Is that guaranteed? */
diff -puN kernel/sched.c~smp_processor_id-cleanup kernel/sched.c
--- 25/kernel/sched.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/kernel/sched.c	2005-06-18 22:32:07.000000000 -0700
@@ -3814,7 +3814,7 @@ EXPORT_SYMBOL(yield);
  */
 void __sched io_schedule(void)
 {
-	struct runqueue *rq = &per_cpu(runqueues, _smp_processor_id());
+	struct runqueue *rq = &per_cpu(runqueues, raw_smp_processor_id());
 
 	atomic_inc(&rq->nr_iowait);
 	schedule();
@@ -3825,7 +3825,7 @@ EXPORT_SYMBOL(io_schedule);
 
 long __sched io_schedule_timeout(long timeout)
 {
-	struct runqueue *rq = &per_cpu(runqueues, _smp_processor_id());
+	struct runqueue *rq = &per_cpu(runqueues, raw_smp_processor_id());
 	long ret;
 
 	atomic_inc(&rq->nr_iowait);
diff -puN kernel/stop_machine.c~smp_processor_id-cleanup kernel/stop_machine.c
--- 25/kernel/stop_machine.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/kernel/stop_machine.c	2005-06-18 22:32:07.000000000 -0700
@@ -100,7 +100,7 @@ static int stop_machine(void)
 	stopmachine_state = STOPMACHINE_WAIT;
 
 	for_each_online_cpu(i) {
-		if (i == _smp_processor_id())
+		if (i == raw_smp_processor_id())
 			continue;
 		ret = kernel_thread(stopmachine, (void *)(long)i,CLONE_KERNEL);
 		if (ret < 0)
@@ -182,7 +182,7 @@ struct task_struct *__stop_machine_run(i
 
 	/* If they don't care which CPU fn runs on, bind to any online one. */
 	if (cpu == NR_CPUS)
-		cpu = _smp_processor_id();
+		cpu = raw_smp_processor_id();
 
 	p = kthread_create(do_stop, &smdata, "kstopmachine");
 	if (!IS_ERR(p)) {
diff -puN lib/kernel_lock.c~smp_processor_id-cleanup lib/kernel_lock.c
--- 25/lib/kernel_lock.c~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/lib/kernel_lock.c	2005-06-18 22:32:07.000000000 -0700
@@ -9,61 +9,6 @@
 #include <linux/module.h>
 #include <linux/kallsyms.h>
 
-#if defined(CONFIG_PREEMPT) && defined(__smp_processor_id) && \
-		defined(CONFIG_DEBUG_PREEMPT)
-
-/*
- * Debugging check.
- */
-unsigned int smp_processor_id(void)
-{
-	unsigned long preempt_count = preempt_count();
-	int this_cpu = __smp_processor_id();
-	cpumask_t this_mask;
-
-	if (likely(preempt_count))
-		goto out;
-
-	if (irqs_disabled())
-		goto out;
-
-	/*
-	 * Kernel threads bound to a single CPU can safely use
-	 * smp_processor_id():
-	 */
-	this_mask = cpumask_of_cpu(this_cpu);
-
-	if (cpus_equal(current->cpus_allowed, this_mask))
-		goto out;
-
-	/*
-	 * It is valid to assume CPU-locality during early bootup:
-	 */
-	if (system_state != SYSTEM_RUNNING)
-		goto out;
-
-	/*
-	 * Avoid recursion:
-	 */
-	preempt_disable();
-
-	if (!printk_ratelimit())
-		goto out_enable;
-
-	printk(KERN_ERR "BUG: using smp_processor_id() in preemptible [%08x] code: %s/%d\n", preempt_count(), current->comm, current->pid);
-	print_symbol("caller is %s\n", (long)__builtin_return_address(0));
-	dump_stack();
-
-out_enable:
-	preempt_enable_no_resched();
-out:
-	return this_cpu;
-}
-
-EXPORT_SYMBOL(smp_processor_id);
-
-#endif /* PREEMPT && __smp_processor_id && DEBUG_PREEMPT */
-
 #ifdef CONFIG_PREEMPT_BKL
 /*
  * The 'big kernel semaphore'
diff -puN lib/Makefile~smp_processor_id-cleanup lib/Makefile
--- 25/lib/Makefile~smp_processor_id-cleanup	2005-06-18 22:32:07.000000000 -0700
+++ 25-akpm/lib/Makefile	2005-06-18 22:32:07.000000000 -0700
@@ -19,6 +19,7 @@ lib-$(CONFIG_RWSEM_GENERIC_SPINLOCK) += 
 lib-$(CONFIG_RWSEM_XCHGADD_ALGORITHM) += rwsem.o
 lib-$(CONFIG_GENERIC_FIND_NEXT_BIT) += find_next_bit.o
 obj-$(CONFIG_LOCK_KERNEL) += kernel_lock.o
+obj-$(CONFIG_DEBUG_PREEMPT) += smp_processor_id.o
 
 ifneq ($(CONFIG_HAVE_DEC_LOCK),y) 
   lib-y += dec_and_lock.o
diff -puN /dev/null lib/smp_processor_id.c
--- /dev/null	2003-09-15 06:40:47.000000000 -0700
+++ 25-akpm/lib/smp_processor_id.c	2005-06-18 22:32:07.000000000 -0700
@@ -0,0 +1,55 @@
+/*
+ * lib/kernel_lock.c
+ *
+ * DEBUG_PREEMPT variant of smp_processor_id.
+ */
+#include <linux/module.h>
+#include <linux/kallsyms.h>
+
+unsigned int debug_smp_processor_id(void)
+{
+	unsigned long preempt_count = preempt_count();
+	int this_cpu = raw_smp_processor_id();
+	cpumask_t this_mask;
+
+	if (likely(preempt_count))
+		goto out;
+
+	if (irqs_disabled())
+		goto out;
+
+	/*
+	 * Kernel threads bound to a single CPU can safely use
+	 * smp_processor_id():
+	 */
+	this_mask = cpumask_of_cpu(this_cpu);
+
+	if (cpus_equal(current->cpus_allowed, this_mask))
+		goto out;
+
+	/*
+	 * It is valid to assume CPU-locality during early bootup:
+	 */
+	if (system_state != SYSTEM_RUNNING)
+		goto out;
+
+	/*
+	 * Avoid recursion:
+	 */
+	preempt_disable();
+
+	if (!printk_ratelimit())
+		goto out_enable;
+
+	printk(KERN_ERR "BUG: using smp_processor_id() in preemptible [%08x] code: %s/%d\n", preempt_count(), current->comm, current->pid);
+	print_symbol("caller is %s\n", (long)__builtin_return_address(0));
+	dump_stack();
+
+out_enable:
+	preempt_enable_no_resched();
+out:
+	return this_cpu;
+}
+
+EXPORT_SYMBOL(debug_smp_processor_id);
+
_
